package com.mh.user.serialport;

import com.mh.user.constants.Constant;
import com.mh.user.utils.CacheTools;
import com.mh.user.utils.ExchangeStringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import purejavacomm.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.TooManyListenersException;

/**
 * @Author : Rainbow
 * @date : 2023/5/12
 */
@Component
@Slf4j
public class SerialTool {

    /**公司电表波特率*/
    public static final int COMPANY_BIT_RATE = 1200;
    /**主机波特率*/
    public static final int CHILLER_BIT_RATE = 9600;
    /**现场电表等设备波特率-待定*/
    public static final int DEVICE_BIT_RATE = 2400;
    /**现场冷量表设备波特率-待定*/
    public static final int CLOUD_BIT_RATE = 9600;
    /**DDC波特率*/
    public static final int DDC_BIT_RATE = 9600;
    /**数据位*/
    public static final int DATA_BITS = SerialPort.DATABITS_8;
    public static final int STOP_BIT = SerialPort.STOPBITS_1;
    /**公司电表校验位-偶校验*/
    public static final int COMPANY_PARITY_BIT = SerialPort.PARITY_EVEN;
    /**冷量表校验位-偶校验*/
    public static final int CLOUD_PARITY_BIT = SerialPort.PARITY_NONE;
    /**通用校验位，无校验*/
    public static final int COMMON_PARITY_BIT = SerialPort.PARITY_NONE;

    /**
     * 查询并返回串口集合
     *
     * @return
     */
    public static List<String> findPorts() {
        Enumeration<CommPortIdentifier> portList = CommPortIdentifier.getPortIdentifiers();
        ArrayList<String> portNameList = new ArrayList<>();
        while (portList.hasMoreElements()) {
            String portName = portList.nextElement().getName();
            portNameList.add(portName);
        }
        return portNameList;
    }

    /**
     * 打开串口
     *
     * @param portName 端口名称
     * @param baudRate 波特率
     * @param dataBits 数据位
     * @param stopBits 停止位
     * @param parity   校验位
     * @return
     */
    public static SerialPort openPort(String portName, Integer baudRate, Integer dataBits, Integer stopBits, Integer parity) throws Exception {
        try {
            //通过端口名识别端口
            CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier(portName);

            //打开串口，设置超时时间
            CommPort commPort = portIdentifier.open(portName, 2000);

            if (commPort instanceof SerialPort) {
                SerialPort serialPort = (SerialPort) commPort;
                try {
                    serialPort.setSerialPortParams(baudRate, dataBits, stopBits, parity);
                    log.info("串口: {} 打开成功",portName);
                } catch (UnsupportedCommOperationException e) {
                    log.error("串口: {} 设置失败,错误原因:{}",portName,e.getMessage());
                    throw e;
                }
                return serialPort;
            }

        } catch (NoSuchPortException e) {
            log.error("无此串口:",e);
            throw e;
        } catch (PortInUseException e) {
            log.error("串口使用中:",e);
            throw e;
        }
        return null;
    }


    public static byte[] HexString2Bytes(String src) {
        if (null == src || 0 == src.length()) {
            return null;
        }
        byte[] ret = new byte[src.length() / 2];
        byte[] tmp = src.getBytes();
        for (int i = 0; i < (tmp.length / 2); i++) {
            ret[i] = uniteBytes(tmp[i * 2], tmp[i * 2 + 1]);
        }
        return ret;
    }

    /**byte转十六进制字符串*/
    public static String ByteArrayToString(byte[] by) {
        String str = "";
        for (int i = 0; i < by.length; i++) {
            String hex = Integer.toHexString(by[i] & 0xFF);
            if (hex.length() == 1) {
                hex = "0" + hex;
            }
            str += hex.toUpperCase();
        }
        return str;
    }

    /**byte类型数据，转成十六进制形式*/
    public static byte uniteBytes(byte src0, byte src1) {
        byte _b0 = Byte.decode("0x" + new String(new byte[] { src0 })).byteValue();
        _b0 = (byte) (_b0 << 4);
        byte _b1 = Byte.decode("0x" + new String(new byte[] { src1 })).byteValue();
        byte ret = (byte) (_b0 ^ _b1);
        return ret;
    }

    /**
     * 关闭串口
     *
     * @throws IOException
     */
    public static synchronized void closePort(SerialPort serialPort) throws IOException {
        if (serialPort != null) {
            serialPort.close();
            log.info("串口" + serialPort.getName() + "已关闭");
        }
    }

    /**
     * 往串口发送数据
     *
     * @param order
     *            待发送数据
     * @throws  IOException
     *             向串口发送数据失败
     *             关闭串口对象的输出流出错
     */
    public static void sendToPort(byte[] order, SerialPort serialPort,String cmd ,String name) throws IOException {

        OutputStream out = null;

        try {

            out = serialPort.getOutputStream();
            out.write(order);
            out.flush();
            log.info(name + "发送数据成功,指令:"+cmd + "  " + serialPort.getName());
        } catch (IOException e) {
            log.error("发送数据失败" + serialPort.getName());
            throw e;
        } finally {
            try {
                if (out != null) {
                    out.close();
                    out = null;
                }
            } catch (IOException e) {
                log.error("关闭串口对象的输出流出错");
                throw e;
            }
        }

    }

    /**
     * 从串口读取数据
     * @param serialPort
     * 当前已建立连接的SerialPort对象
     * @return 读取到的数据
     * @throws IOException
     * 从串口读取数据时出错
     * @throws IOException
     * 关闭串口对象输入流出错
     */
    public static byte[] readFromPort(SerialPort serialPort) throws Exception {

        InputStream in = null;
        byte[] bytes = null;

        try {
            if (serialPort != null) {
                in = serialPort.getInputStream();
            } else {
                return null;
            }
            int bufflenth = in.available(); // 获取buffer里的数据长度
            while (bufflenth != 0) {
                bytes = new byte[bufflenth]; // 初始化byte数组为buffer中数据的长度
                in.read(bytes);
                bufflenth = in.available();
            }
            if (bytes == null) {
                return null;
            }
        } catch (Exception e) {
            throw e;
        } finally {
            try {
                if (in != null) {
                    in.close();
                    in = null;
                }
            } catch (IOException e) {
                throw e;
            }

        }
        return bytes;

    }

    /**
     * 添加监听器
     *
     * @param serialPort
     *            串口对象
     * @param listener
     *            串口监听器
     * @throws TooManyListenersException
     *             监听类对象过多
     */
    public static void addListener(SerialPortEventListener listener, SerialPort serialPort) throws TooManyListenersException {

        try {
            //判断监听器是否存在
            // 给串口添加监听器
            serialPort.addEventListener(listener);
            // 设置当有数据到达时唤醒监听接收线程
            serialPort.notifyOnDataAvailable(true);
            // 设置当通信中断时唤醒中断线程
            serialPort.notifyOnBreakInterrupt(true);

        } catch (TooManyListenersException e) {
            throw e;
        }
    }

    private static String byteToString (byte[] bytes) {
        if (null == bytes || bytes.length == 0) {
            return "";
        }
        String strContent = "";
        try {
            strContent = new String(bytes, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return strContent;
    }


    //public static void main(String[] args) {
    //    List<String> ports = findPorts();
    //    String str = "FEFEFE6821034010010968010235189E16";
    //    byte[] cmd = HexString2Bytes(str);
    //    System.out.println("ports = " + ports);
    //    try {
    //        SerialPort serialPort = SerialTool.openPort(ports.get(1), COMPANY_BIT_RATE, DATA_BITS, STOP_BIT, COMPANY_PARITY_BIT);
    //        SerialTool.sendToPort(cmd,serialPort);
    //        Thread.sleep(2000);
    //        byte[] bytes = SerialTool.readFromPort(serialPort);
    //        String receive = byteToString(bytes);
    //        String receive2 = ByteArrayToString(bytes);
    //        System.out.println("receive = " + receive);
    //        System.out.println("receive2 = " + receive2);
    //        //SerialTool.closePort(serialPort);
    //    } catch (Exception e) {
    //        e.printStackTrace();
    //    }
    //
    //}
}
